from random import randint
from typing import ClassVar
from unittest import TestCase
from unittest.mock import patch


class Dragon:
    HEALTH_MIN: ClassVar[int] = 50
    HEALTH_MAX: ClassVar[int] = 100
    name: str
    health: int

    def __init__(self, name: str, /) -> None:
        self.name = name
        self.health = randint(self.HEALTH_MIN, self.HEALTH_MAX)


class TestName(TestCase):
    def test_name_default(self):
        with self.assertRaises(TypeError):
            dragon = Dragon()  # noqa

    def test_name_positional(self):
        dragon = Dragon('MyName')
        self.assertEqual(dragon.name, 'MyName')

    def test_name_keyword(self):
        with self.assertRaises(TypeError):
            dragon = Dragon(name='MyName')  # noqa


class TestHealth(TestCase):
    def setUp(self):
        self.dragon = Dragon('MyName')

    def test_health_default(self):
        self.assertGreaterEqual(self.dragon.health, 50)
        self.assertLessEqual(self.dragon.health, 100)

    def test_health_default_many(self):
        for _ in range(10_000):
            self.dragon = Dragon('MyName')
            self.assertGreaterEqual(self.dragon.health, 50)
            self.assertLessEqual(self.dragon.health, 100)

    def test_health_default_patch(self):
        with patch('dragon.randint', return_value=74) as randint:  # noqa
            dragon = Dragon('MyName')
            self.assertEqual(dragon.health, 74)

    def test_health_hasattr_HEALTH_MIN(self):
        self.assertTrue(hasattr(Dragon, 'HEALTH_MIN'))
        self.assertEqual(Dragon.HEALTH_MIN, 50)

    def test_health_hasattr_HEALTH_MAX(self):
        self.assertTrue(hasattr(Dragon, 'HEALTH_MAX'))
        self.assertEqual(Dragon.HEALTH_MAX, 100)
